@@ -96,6 +96,8 @@ void errorf(const char *fmt, ...) {
 
  gtk_dialog_run(GTK_DIALOG(dialog));
   gtk_widget_destroy(dialog);
+
+  free(buf);
 }
 
 gpx_t *choose_file(appdata_t *appdata, gboolean whole_dir) {
@@ -1204,6 +1206,12 @@ cb_menu_paste(GtkWidget *widget, gpointer data) {
     printf("paste: ERROR, no active buffer\n");
 }
 
+static void 
+cb_menu_export_log(GtkWidget *widget, gpointer data) {
+  appdata_t *appdata = (appdata_t*)data;
+  notes_log_export(appdata);
+}
+
 static void 
 cb_menu_export_mmpoi(GtkWidget *widget, gpointer data) {
   appdata_t *appdata = (appdata_t*)data;
@@ -1220,21 +1228,18 @@ void menu_create(appdata_t *appdata) {
   GtkWidget *menu, *item;  
   menu = gtk_menu_new();
 
+#ifdef USE_MAEMO
+  appdata->menu_import = 
+#endif
   item = gtk_menu_item_new_with_label(_("Import"));
   gtk_menu_append(GTK_MENU_SHELL(menu), item);
   GtkWidget *submenu = gtk_menu_new();
   gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), submenu);
 
-#ifdef USE_MAEMO
-  appdata->menu_add = 
-#endif
   item = gtk_menu_item_new_with_label( _("Add file...") );
   gtk_menu_append(GTK_MENU_SHELL(submenu), item);
   g_signal_connect(item, "activate", GTK_SIGNAL_FUNC(cb_menu_add), appdata);
 
-#ifdef USE_MAEMO
-  appdata->menu_add_dir = 
-#endif
   item = gtk_menu_item_new_with_label( _("Add dir...") );
   gtk_menu_append(GTK_MENU_SHELL(submenu), item);
   g_signal_connect(item, "activate", GTK_SIGNAL_FUNC(cb_menu_adddir), appdata);
@@ -1244,19 +1249,24 @@ void menu_create(appdata_t *appdata) {
   gtk_menu_append(GTK_MENU_SHELL(submenu), item);
   g_signal_connect(item, "activate", GTK_SIGNAL_FUNC(cb_menu_remove), appdata);
 
-  item = gtk_menu_item_new_with_label(_("Export"));
+#ifdef USE_MAEMO
+  appdata->menu_export = 
+#endif
+    item = gtk_menu_item_new_with_label(_("Export"));
   gtk_menu_append(GTK_MENU_SHELL(menu), item);
   submenu = gtk_menu_new();
   gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), submenu);
 
-#ifdef USE_MAEMO
-  appdata->menu_export_mmpoi = 
-#endif
   item = gtk_menu_item_new_with_label( _("Maemo Mapper POI...") );
   gtk_menu_append(GTK_MENU_SHELL(submenu), item);
   g_signal_connect(item, "activate", 
 		   GTK_SIGNAL_FUNC(cb_menu_export_mmpoi), appdata);
 
+  item = gtk_menu_item_new_with_label( _("Garmin Field Notes...") );
+  gtk_menu_append(GTK_MENU_SHELL(submenu), item);
+  g_signal_connect(item, "activate", 
+		   GTK_SIGNAL_FUNC(cb_menu_export_log), appdata);
+
   item = gtk_menu_item_new_with_label( _("Garmin GPX...") );
   gtk_menu_append(GTK_MENU_SHELL(submenu), item);
   g_signal_connect(item, "activate", 
@@ -1514,6 +1524,7 @@ crumb_back(gpointer data) {
 
     if(!gpx) {
       gtk_widget_set_sensitive(crumb->appdata->menu_search,  TRUE);
+      gtk_widget_set_sensitive(crumb->appdata->menu_export,  TRUE);
       printf("no search data found, return to gpx\n");
       gpx = crumb->appdata->cur_gpx;
     } else
@@ -1570,9 +1581,9 @@ crumb_back(gpointer data) {
     }
 
     /* enable gpxlist related menu entries */
-    gtk_widget_set_sensitive(crumb->appdata->menu_add,     TRUE);
-    gtk_widget_set_sensitive(crumb->appdata->menu_add_dir, TRUE);
+    gtk_widget_set_sensitive(crumb->appdata->menu_import,  TRUE);
     gtk_widget_set_sensitive(crumb->appdata->menu_search,  TRUE);
+    gtk_widget_set_sensitive(crumb->appdata->menu_export,  TRUE);
   }
 
   gtk_widget_show_all(crumb->appdata->vbox);    
@@ -1605,12 +1616,12 @@ static void crumb_add(appdata_t *appdata, char *name, int level,
     appdata->cur_cache = (cache_t*)user_data;
     /* the cache view cannot be searched */
     gtk_widget_set_sensitive(appdata->menu_search,  FALSE);
+    gtk_widget_set_sensitive(appdata->menu_export,  FALSE);
   }
 
   if(level != CRUMB_GPXLIST) {
     /* disable gpxlist related menu entries */
-    gtk_widget_set_sensitive(appdata->menu_add,     FALSE);
-    gtk_widget_set_sensitive(appdata->menu_add_dir, FALSE);
+    gtk_widget_set_sensitive(appdata->menu_import,  FALSE);
     gtk_widget_set_sensitive(appdata->menu_remove,  FALSE);
   }
 
