@@ -20,6 +20,8 @@
 #include <stdio.h>
 #include <string.h>
 
+#include <glib/gunicode.h>
+
 #include <libxml/parser.h>
 #include <libxml/tree.h>
 
@@ -94,6 +96,11 @@ static notes_t *notes_load(appdata_t *appdata, cache_t *cache) {
 		xmlFree(str);
 	      }
 
+	      if((str = xmlGetProp(wpt_node, "logged"))) {
+		notes->logged = (strcasecmp(str, "yes") == 0);
+		xmlFree(str);
+	      }
+
 	      if((str = xmlGetProp(wpt_node, "lat"))) {
 		notes->pos.lat = g_ascii_strtod(str, NULL);
 		xmlFree(str);
@@ -147,7 +154,8 @@ void notes_load_all(appdata_t *appdata, gpx_t *gpx) {
 
 static int notes_save(cache_context_t *context,
 		      char *text, pos_t pos, 
-		      gboolean override, gboolean found, time_t ftime) {
+		      gboolean override, gboolean found, 
+		      time_t ftime, gboolean logged) {
   /* build local path */
   int path_len = strlen(context->appdata->image_path) +
     2 * strlen(context->cache->id) + 6;
@@ -176,6 +184,11 @@ static int notes_save(cache_context_t *context,
     else
       xmlNewProp(wpt_node, BAD_CAST "override", BAD_CAST "no");
 
+    if(logged)
+      xmlNewProp(wpt_node, BAD_CAST "logged", BAD_CAST "yes");
+    else
+      xmlNewProp(wpt_node, BAD_CAST "logged", BAD_CAST "no");
+
     if(found) {
       if(ftime) {
 	char str[32];
@@ -240,6 +253,8 @@ gint notes_destroy_event(GtkWidget *widget, gpointer data ) {
       gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(context->notes.overridew));
     gboolean found = 
       gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(context->notes.foundw));
+    gboolean logged =
+      gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(context->notes.loggedw));
 
     /* required accuracy is 1/60000 degree */
     if(((int)(pos.lat * 60000 + .5) != 
@@ -259,7 +274,7 @@ gint notes_destroy_event(GtkWidget *widget, gpointer data ) {
 	(int)(context->cache->pos.lat * 60000 + .5)) &&
        ((int)(pos.lon * 60000 + .5) == 
 	(int)(context->cache->pos.lon * 60000 + .5)) &&
-       !override && !found && (strlen(text) == 0)) {
+       !override && !found && !logged && (strlen(text) == 0)) {
       printf("notes are in default state, removing them if present\n");
 
       /* remove note */
@@ -313,7 +328,8 @@ gint notes_destroy_event(GtkWidget *widget, gpointer data ) {
       /* - update the notes entry in the loaded gpx tree */
       
       /* update file on disk */
-      notes_save(context, text, pos, override, found, context->notes.ftime);
+      notes_save(context, text, pos, override, found, 
+		 context->notes.ftime, logged);
     
       /* search for matching caches and replace note there */
       notes_t *note = NULL;
@@ -334,6 +350,7 @@ gint notes_destroy_event(GtkWidget *widget, gpointer data ) {
 	    cache->notes->pos = pos;
 	    cache->notes->override = override;
 	    cache->notes->found = found;
+	    cache->notes->logged = logged;
 	    cache->notes->ftime = context->notes.ftime;
 	  }
 	  cache = cache->next;
@@ -406,7 +423,6 @@ static void ftime_update(GtkWidget *widget, cache_context_t *context,
       strftime(str, sizeof(str), "%x %X", loctime);
       
       gtk_label_set_text(GTK_LABEL(context->notes.datew), str);
-      gtk_label_attrib_set(context->notes.datew, str, SIZE_SMALL, 0);
     } else
       gtk_label_set_text(GTK_LABEL(context->notes.datew), "");
 
@@ -446,6 +462,33 @@ static void callback_modified(GtkWidget *widget, gpointer data ) {
 
     ftime_update(widget, context, TRUE);
   }
+
+  if(widget == context->notes.loggedw) {
+    printf("was loggedw\n");
+
+    /* about to remove "found" flag -> ask for confirmation */
+    if(!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget))) {
+      GtkWidget *dialog = gtk_message_dialog_new(
+		 GTK_WINDOW(context->appdata->window),
+		 GTK_DIALOG_DESTROY_WITH_PARENT,
+		 GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO,
+		 _("Do you really want to remove the \"logged\" flag? "
+		   "This may cause problems on your next Garmin Field "
+		   "Notes upload!"));
+      
+      gtk_window_set_title(GTK_WINDOW(dialog), _("Reset \"logged\" flag?"));
+
+      /* set the active flag again if the user answered "no" */
+      if(GTK_RESPONSE_NO == gtk_dialog_run(GTK_DIALOG(dialog)))
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), TRUE);
+      else {
+	gtk_widget_set_sensitive(widget, FALSE);
+	gtk_widget_set_sensitive(context->notes.foundw, TRUE);
+      }
+
+      gtk_widget_destroy(dialog);
+    }
+  }
 }
 
 static gboolean focus_in(GtkWidget *widget, GdkEventFocus *event,
@@ -515,11 +558,25 @@ GtkWidget *cache_notes(cache_context_t *context) {
   gtk_table_attach_defaults(GTK_TABLE(table),
 			    context->notes.overridew, 0, 1, 1, 2);  
 
+  GtkWidget *hbox = gtk_hbox_new(FALSE, 2);
+
   context->notes.foundw = gtk_check_button_new_with_label(_("Found"));
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(context->notes.foundw), 
 			       cache->notes && cache->notes->found);
-  gtk_table_attach_defaults(GTK_TABLE(table),
-			    context->notes.foundw, 3, 4, 0, 1);  
+  gtk_box_pack_start_defaults(GTK_BOX(hbox), context->notes.foundw);
+
+  context->notes.loggedw = gtk_check_button_new_with_label(_("Logged"));
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(context->notes.loggedw), 
+			       cache->notes && cache->notes->logged);
+  gtk_box_pack_start_defaults(GTK_BOX(hbox), context->notes.loggedw);
+  
+  gtk_table_attach_defaults(GTK_TABLE(table), hbox, 3, 4, 0, 1);  
+
+  /* only found and logged caches have a log flag the user can change */
+  if(!(cache->notes && cache->notes->found && cache->notes->logged))
+    gtk_widget_set_sensitive(context->notes.loggedw, FALSE);
+  else
+    gtk_widget_set_sensitive(context->notes.foundw, FALSE);
 
   context->notes.datew = gtk_label_new("");
   gtk_misc_set_alignment(GTK_MISC(context->notes.datew), 0.0f, 0.5f);
@@ -529,19 +586,14 @@ GtkWidget *cache_notes(cache_context_t *context) {
 
   pos_t pos = gpx_cache_pos(cache);
   if(cache->notes) pos = cache->notes->pos;
-    
-  gtk_table_attach_defaults(GTK_TABLE(table),
-			    label = gtk_label_new(_("Latitude:")), 1, 2, 0, 1);
-  gtk_misc_set_alignment(GTK_MISC(label), 1.f, 0.5f);
+
   gtk_table_attach_defaults(GTK_TABLE(table), 
 	    context->notes.latw = lat_entry_new(pos.lat), 2, 3, 0, 1);
-  gtk_table_attach_defaults(GTK_TABLE(table),
-			    label = gtk_label_new(_("Longitude:")), 1, 2, 1, 2);
-  gtk_misc_set_alignment(GTK_MISC(label), 1.f, 0.5f);
+
   gtk_table_attach_defaults(GTK_TABLE(table), 
 	    context->notes.lonw = lon_entry_new(pos.lon), 2, 3, 1, 2);
 
-  GtkWidget *hbox = gtk_hbox_new(FALSE, 0);
+  hbox = gtk_hbox_new(FALSE, 0);
   gtk_box_pack_start(GTK_BOX(hbox), table, FALSE, FALSE, 0);  
   gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);  
   
@@ -594,6 +646,8 @@ GtkWidget *cache_notes(cache_context_t *context) {
 		   G_CALLBACK(callback_modified), context);
   g_signal_connect(G_OBJECT(context->notes.foundw), "toggled",
 		   G_CALLBACK(callback_modified), context);
+  g_signal_connect(G_OBJECT(context->notes.loggedw), "toggled",
+		   G_CALLBACK(callback_modified), context);
 
   return vbox;
 }
@@ -621,3 +675,300 @@ gboolean notes_get_override(cache_context_t *context) {
 		      GTK_TOGGLE_BUTTON(context->notes.overridew));
 
 }
+
+typedef struct {
+  GtkWidget *info_label;
+  GtkWidget *dialog;
+  appdata_t *appdata;
+  GtkWidget *path_label;
+} export_context_t;
+
+static void on_browse(GtkWidget *widget, gpointer data) {
+  GtkWidget *dialog;
+
+  export_context_t *context = (export_context_t*)data;
+
+#ifdef USE_MAEMO
+  dialog = hildon_file_chooser_dialog_new(GTK_WINDOW(context->dialog), 
+					  GTK_FILE_CHOOSER_ACTION_SAVE);
+#else
+  dialog = gtk_file_chooser_dialog_new(_("Save POI database"),
+				       GTK_WINDOW(context->dialog),
+				       GTK_FILE_CHOOSER_ACTION_SAVE,
+				       GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+				       GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
+				       NULL);
+#endif
+
+  printf("set filename <%s>\n", context->appdata->fieldnotes_path);
+
+  if(!g_file_test(context->appdata->fieldnotes_path, G_FILE_TEST_EXISTS)) {
+    char *last_sep = strrchr(context->appdata->fieldnotes_path, '/');
+    if(last_sep) {
+      *last_sep = 0;  // seperate path from file 
+
+      /* the user just created a new document */
+      gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dialog), 
+					  context->appdata->fieldnotes_path);
+      gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(dialog), last_sep+1);
+
+      /* restore full filename */
+      *last_sep = '/';
+    }
+  } else 
+    gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(dialog), 
+				  context->appdata->fieldnotes_path);
+
+  if (gtk_dialog_run (GTK_DIALOG(dialog)) == GTK_FM_OK) {
+    gchar *name = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
+    if(name) {
+      free(context->appdata->fieldnotes_path);
+      context->appdata->fieldnotes_path = strdup(name);
+      gtk_label_set_text(GTK_LABEL(context->path_label), 
+			 context->appdata->fieldnotes_path);
+    }
+  }
+  
+  gtk_widget_destroy (dialog);
+}
+
+typedef struct log_chain_s {
+  cache_t *cache;
+  struct log_chain_s *next;
+} log_chain_t;
+
+void notes_log_export(appdata_t *appdata) {
+  gpx_t *gpx = appdata->gpx;
+  log_chain_t *log = NULL, *llog, **clog = &log;
+
+  export_context_t context;
+  memset(&context, 0, sizeof(export_context_t));
+  context.appdata = appdata;
+
+  printf("export log\n");
+
+  int logs2export = 0;
+
+  /* make sure all notes are loaded */
+  while(gpx) {
+    if(!gpx->notes_loaded) {
+      notes_load_all(appdata, gpx);
+      gpx->notes_loaded = TRUE;
+    }
+
+    cache_t *cache = gpx->cache;
+    while(cache) {
+      if(cache->notes && cache->notes->found && !cache->notes->logged) {
+	gboolean already_chained = FALSE;
+	llog = log;
+	while(llog) {
+	  if(strcasecmp(llog->cache->id, cache->id) == 0)
+	    already_chained = TRUE;
+
+	  llog = llog->next;
+	}
+
+	if(!already_chained) {
+	  logs2export++;
+
+	  printf("chaining log for %s\n", cache->id);
+
+	  *clog = g_new0(log_chain_t, 1);
+	  (*clog)->cache = cache; 
+	  clog = &((*clog)->next);
+	} else
+	  printf("dup for %s\n", cache->id);
+	
+      }
+
+      cache = cache->next;
+    }
+
+    gpx = gpx->next;
+  }
+
+
+  /* ------------- confirmation dialog ---------------- */
+  char *old_fieldnotes_path = strdup(appdata->fieldnotes_path);
+
+  context.dialog = gtk_dialog_new_with_buttons(_("Garmin Field Notes Export"),
+	  GTK_WINDOW(appdata->window), GTK_DIALOG_MODAL,
+	  GTK_STOCK_OK, GTK_RESPONSE_ACCEPT,
+          GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT,
+          NULL);
+
+#ifdef USE_MAEMO
+  hildon_help_dialog_help_enable(GTK_DIALOG(context.dialog), 
+		 HELP_ID_FIELDNOTES, appdata->osso_context);
+#endif 
+
+  GtkWidget *vbox = gtk_vbox_new(FALSE,0);
+
+  /* ------------------ info text -------------- */
+
+  char *msg = g_strdup_printf(_("This will export the notes of %d caches "
+				"into the given file. This file can be "
+				"uploaded to geocaching.com for logging."),
+			      logs2export);
+
+  context.info_label = gtk_label_new(msg);
+  g_free(msg);
+  gtk_label_set_line_wrap_mode(GTK_LABEL(context.info_label), PANGO_WRAP_WORD);
+  gtk_label_set_line_wrap(GTK_LABEL(context.info_label), TRUE);
+  gtk_misc_set_alignment(GTK_MISC(context.info_label), 0.f, 0.5f);
+  gtk_box_pack_start_defaults(GTK_BOX(vbox), context.info_label);
+
+  /* ------------------ path/file ------------------ */
+  gtk_box_pack_start_defaults(GTK_BOX(vbox), gtk_hseparator_new());
+
+  GtkWidget *hbox = gtk_hbox_new(FALSE, 0);
+  GtkWidget *label = gtk_label_new(_("Export to:"));
+  gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE,0);
+  gtk_misc_set_alignment(GTK_MISC(label), 0.f, 0.5f);
+  GtkWidget *button = gtk_button_new_with_label(_("Browse..."));
+  gtk_signal_connect(GTK_OBJECT(button), "clicked",
+		     GTK_SIGNAL_FUNC(on_browse), (gpointer)&context);
+  gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE,0);
+  gtk_box_pack_start_defaults(GTK_BOX(vbox), hbox);
+
+  context.path_label = gtk_label_new(appdata->fieldnotes_path);
+  gtk_misc_set_alignment(GTK_MISC(context.path_label), 0.f, 0.5f);
+  gtk_label_set_ellipsize(GTK_LABEL(context.path_label), 
+			  PANGO_ELLIPSIZE_MIDDLE);
+  gtk_box_pack_start_defaults(GTK_BOX(vbox), context.path_label);
+
+  label = gtk_label_new(_("(a %s in the filename will be replaced by the "
+			  "current date and time)"));
+  gtk_label_set_line_wrap_mode(GTK_LABEL(label), PANGO_WRAP_WORD);
+  gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
+  gtk_misc_set_alignment(GTK_MISC(label), 0.f, 0.5f);
+  gtk_box_pack_start_defaults(GTK_BOX(vbox), label);
+
+  /* ------------------ info ------------------ */
+
+  gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(context.dialog)->vbox), vbox);
+
+  gtk_widget_show_all(context.dialog);
+
+  if(GTK_RESPONSE_ACCEPT == gtk_dialog_run(GTK_DIALOG(context.dialog))) {
+
+    /* ---------------- do actual export ------------------ */
+
+    time_t now = time(NULL);
+    struct tm *tm_now = localtime(&now);
+    char now_str[32];
+    strftime(now_str, sizeof(now_str)-1, "%F_%H:%M", tm_now);
+    char *fname = g_strdup_printf(appdata->fieldnotes_path, now_str);
+    printf("--- about to export logs to %s ---\n", fname);
+    FILE *file = fopen(fname, "w");
+    g_free(fname);
+
+    if(file) {
+
+      llog = log;
+      while(llog) {
+	printf("Exporting %s\n", llog->cache->id);
+
+	/* ----------- build utc time string ----------- */
+	
+	char *tz = getenv("TZ");
+	setenv("TZ", "", 1);
+	tzset();
+	struct tm *tm = localtime(&llog->cache->notes->ftime);
+	char tstr[32];
+	strftime(tstr, sizeof(tstr)-1, "%FT%H:%MZ", tm);
+	if(tz) setenv("TZ", tz, 1);
+	else   unsetenv("TZ");
+	tzset();
+
+	/* escape \" in text */
+	char *text = NULL;
+	if(llog->cache->notes->text) {
+	  text = g_strdup(llog->cache->notes->text);
+	  char *p = text;
+	  while(*p) {
+	    /* convert " to ' */
+	    if(*p == '\"') *p = '\'';
+	    p++;
+	  }
+	} else
+	  text = g_strdup("");
+	
+	/* ----------- build complete log string ------------ */
+	char *str = g_strdup_printf("%s,%s,Found it,\"%s\"\n", 
+				    llog->cache->id, tstr, text);
+	g_free(text);
+	
+	/* ----------- convert to unicode ------------ */
+	glong written = 0;
+	gunichar2 *uc = g_utf8_to_utf16(str, -1, NULL, &written, NULL);
+	g_free(str);
+
+	/* -------------- and write it --------------- */
+	fwrite(uc, written, sizeof(gunichar2), file);
+	g_free(uc);
+	
+	/* -------------- set "logged" flag in notes and update them ------ */
+	llog->cache->notes->logged = TRUE;
+
+	/* update flags in all copies of this cache */
+
+	gpx_t *tgpx = appdata->gpx;
+	while(tgpx) {
+	  cache_t *tcache = tgpx->cache;
+	  while(tcache) {
+	    if((tcache != llog->cache) && 
+	       (strcmp(tcache->id, llog->cache->id) == 0)) {
+	      printf("found dup cache %s in %s\n", tcache->id, tgpx->name);
+
+	      if(tcache->notes)
+		notes_free(tcache->notes);
+
+	      /* create a new note for this cache */
+	      tcache->notes = malloc(sizeof(notes_t));
+	      memset(tcache->notes, 0, sizeof(notes_t));
+	      if(llog->cache->notes->text)
+		tcache->notes->text = strdup(llog->cache->notes->text);
+	      tcache->notes->pos = llog->cache->notes->pos;
+	      tcache->notes->override = llog->cache->notes->override;
+	      tcache->notes->found = llog->cache->notes->found;
+	      tcache->notes->logged = llog->cache->notes->logged;
+	      tcache->notes->ftime = llog->cache->notes->ftime;
+	    }
+	    tcache = tcache->next;
+	  }
+	  tgpx = tgpx->next;
+	}
+	
+	/* finally write the notes file itself */
+	cache_context_t ccontext;
+	ccontext.appdata = appdata;
+	ccontext.cache = llog->cache;
+
+	notes_save(&ccontext,
+		   llog->cache->notes->text, llog->cache->notes->pos, 
+		   llog->cache->notes->override, llog->cache->notes->found, 
+		   llog->cache->notes->ftime, llog->cache->notes->logged);
+
+	llog = llog->next;
+      }
+    
+      fclose(file);
+    }
+  } else {
+    /* restore old path, in case it has been altered but not been used */
+    free(appdata->fieldnotes_path);
+    appdata->fieldnotes_path = strdup(old_fieldnotes_path);
+  }
+
+  gtk_widget_destroy(context.dialog);
+  
+  free(old_fieldnotes_path);
+
+  /* free list */
+  while(log) {
+    log_chain_t *next = log->next;
+    g_free(log);
+    log = next;
+  }
+}
